#include "z800.h"
#include "EMADevice.h"

#include <sys/ioctl.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>


//---------------------------------------------------------------------------
EMADevice::EMADevice()
{
	init( NULL, EMA_NONE );
}

//---------------------------------------------------------------------------
EMADevice::EMADevice( EMADeviceInfo * deviceInfo )
{
	init( deviceInfo, EMA_NONE );
}

//---------------------------------------------------------------------------
EMADevice::EMADevice( EMAFlags flags )
{
	init( NULL, flags);
}

//---------------------------------------------------------------------------
EMADevice::EMADevice( EMADeviceInfo * deviceInfo, EMAFlags flags ) :
	m_device(0),
	m_version(0)
{
	init( deviceInfo, flags );
}

//---------------------------------------------------------------------------
void EMADevice::init( EMADeviceInfo * deviceInfo, EMAFlags flags )
{
	// FIXME: don't hard code the device path
	m_device = open( "/dev/z800:0", O_RDWR );
	if( m_device == -1 )
	{
		printf( "EMADevice::InternalConstructor: Error opening z800\n" );
		return;
	}
	
	// get and cache the firmware version of this eMagin HMD
	unsigned char buffer[8];	
	int retValue = ioctl( m_device, IOCTL_GET_FIRMWARE_VERSION, buffer, 8 );
	if( retValue == -1 ) {
		printf( "Error getting firmware version. errno = %d\n", errno );
		close(m_device);
		m_device = 0;
		return;
	}
	
	m_version = buffer[1] + (buffer[0]<<8);
	
	printf( "EMADevice: Z800 Firmware Version: %x.%x\n", buffer[0], buffer[1] );

	// now do a quick wake up of the HMD
	if(KeepAlive())
	{
		printf( "EMADevice:: Failed to wake up Z800!\n" );
	}
}

//---------------------------------------------------------------------------
EMADevice::~EMADevice()
{
	printf( "EMADevice::dtor...\n" );
	if( !(m_device == 0 || m_device == -1 ))
	{
		// close the device
		close(m_device);
		m_device = 0;	
	}
}

//---------------------------------------------------------------------------
int EMADevice::GetDeviceInfo( EMADeviceInfo * devInfos )
{
    return -1;//::LastError ( ::GetDeviceInfo ( this, devInfos ) ) ;
}

//---------------------------------------------------------------------------
int EMADevice::GetVersion( int * version, int * build ) 
{ 
    return -1;//::LastError ( ::GetEMADeviceVersion ( version , build ) ) ; 
} 

//---------------------------------------------------------------------------
int EMADevice::EnumerateDevices( EMADeviceInfo * devInfos, int maxDevices, int * foundDevices ) 
{
    return -1;//::LastError ( ::EnumerateDevices ( devInfos , maxDevices , foundDevices ) ) ;
}

// Mouse and head tracking 
// -----------------------
//

//---------------------------------------------------------------------------
int EMADevice::EnableMouseTracking ( bool enable )
{
    return -1;//::LastError ( ::EnableMouseTracking ( this , enable ? TRUE : FALSE  ) ) ;
}

//---------------------------------------------------------------------------
int EMADevice::ResetMouseTracking()
{
    return -1;//::LastError ( ::ResetMouseTracking ( this ) ) ;
}

//---------------------------------------------------------------------------
int EMADevice::PollHeadTrackerRawData( EMADeviceData * deviceData ) 
{
    return -1;//::LastError ( ::PollHeadTrackerRawData ( this , deviceData ) ) ;
}

// 3D and Misc USB commands 
// ------------------------
//

//---------------------------------------------------------------------------
int EMADevice::EnableExternalMonitor( bool enable )
{
    return -1;//::LastError ( ::EnableExternalMonitor ( this, enable ? TRUE : FALSE ) ) ;
}

//---------------------------------------------------------------------------
int EMADevice::EnableStereoVision( bool enable )
{
	int retValue = -1;
	
	// FIXME: check for 3D capability
	if( m_device )//&& canDo3D() ) 
	{
		retValue = ioctl( m_device, IOCTL_SET_ENABLE_3D, enable?1:0 );
		if( retValue == -1 ) 
		{
			printf( "EMADevice::EnableStereoVision: Error with ioctl\n" );
		}
	}

    return retValue;
}

//---------------------------------------------------------------------------
int EMADevice::GoSleep()
{
	if(m_device)
	{
		int retValue = ioctl( m_device, IOCTL_SLEEP );
		if( retValue == -1 ) 
		{
			printf( "EMADevice::Sleep: Error with ioctl\n" );
			return -1;
		}
	}
 		
    return 0;
}

//---------------------------------------------------------------------------
int EMADevice::KeepAlive()
{
	if(m_device)
	{
		int retValue = ioctl( m_device, IOCTL_KEEPALIVE );
		if( retValue == -1 ) 
		{
			printf( "EMADevice::KeepAlive: Error with ioctl\n" );
			return -1;
		}
	}
}

//---------------------------------------------------------------------------
int EMADevice::GetFirmwareVersion( uint32_t & firmware )
{
	firmware = m_version;
    return 0;
}

//---------------------------------------------------------------------------
int EMADevice::ToggleLeftRightDisplays()
{
    return -1;//::LastError ( ::ToggleLeftRightDisplays ( this ) ) ;
}

//---------------------------------------------------------------------------
int EMADevice::CycleBrightnessLevel()
{
	if( m_device )
	{
		int retValue = ioctl( m_device, IOCTL_CYCLEBRIGHTNESS );
		if( retValue == -1 ) 
		{
			printf( "EMADevice::CycleBrightness: Error with ioctl\n" );
			return -1;
		}
	}
    return 0;
}

//---------------------------------------------------------------------------
int EMADevice::ResetEEPROM()
{
    return -1;//::LastError ( ::ResetEEPROM ( this ) ) ;
}

// Properties 
// ----------
//
//---------------------------------------------------------------------------
int EMADevice::GetProperty ( EMADisplayType display , EMAProperty propertyID , int * value ) 
{
    return -1;//::LastError ( ::GetProperty ( this , display , propertyID , value ) ) ;
}

//---------------------------------------------------------------------------
int EMADevice::SetProperty ( EMADisplayType display , EMAProperty propertyID , int value ) 
{
    return -1;//::LastError ( ::SetProperty ( this , display , propertyID , value ) ) ;
}

//---------------------------------------------------------------------------
int EMADevice::GetPropertyFactorySetting ( EMAProperty propertyID , int * propertyDefault )
{
    return -1;//::LastError ( ::GetPropertyFactorySetting ( propertyID , propertyDefault ) ) ;
}

//---------------------------------------------------------------------------
int EMADevice::GetPropertyRange ( EMAProperty propertyID , int * minValue , int * maxValue ) 
{
    return -1;//::LastError ( ::GetPropertyRange ( propertyID , minValue , maxValue ) ) ;
}

//---------------------------------------------------------------------------
int EMADevice::RestorePropertyFactorySetting ( EMADisplayType display , EMAProperty propertyID ) 
{
    return -1;//::LastError ( ::RestorePropertyFactorySetting ( this , display , propertyID ) ) ;
}

//---------------------------------------------------------------------------
int EMADevice::RestoreFactorySettings() 
{
    return -1;//::LastError ( ::RestoreFactorySettings ( this ) ) ;
}

// Gain and Offset 
// ---------------
//
//---------------------------------------------------------------------------
int EMADevice::GetGainOffset( EMADisplayType display, EMAGainOffset * data ) 
{
    return -1;//::LastError ( ::GetGainOffset ( this , display , data ) ) ;
}

//---------------------------------------------------------------------------
int EMADevice::SetGainOffset( EMADisplayType display, EMAGainOffset * data ) 
{
    return -1;//::LastError ( ::SetGainOffset ( this , display , data ) ) ;
}

//---------------------------------------------------------------------------
int EMADevice::GetGainRange( uint8_t * minGain, uint8_t * maxGain ) 
{
    return -1;//::LastError ( ::GetGainRange ( minGain , maxGain ) ) ;
}

//---------------------------------------------------------------------------
int EMADevice::GetOffsetRange( uint8_t * minOffset, uint8_t * maxOffset ) 
{
    return -1;//::LastError ( ::GetOffsetRange ( minOffset , maxOffset ) ) ;
}

//---------------------------------------------------------------------------
int EMADevice::GetGainOffsetFactorySetting( uint8_t * defaultGain, uint8_t * defaultOffset  ) 
{ 
    return -1;//::LastError ( ::GetGainOffsetFactorySetting  ( defaultGain , defaultOffset ) ) ;
} 

//---------------------------------------------------------------------------
int EMADevice::RestoreGainOffsetFactorySetting( EMADisplayType display ) 
{
    return -1;//::LastError ( ::RestoreGainOffsetFactorySetting ( this , display ) ) ;
} 
