#include "EMADevice.h"

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <strings.h>
#include <GL/gl.h>
#include <GL/glut.h>
#include <GL/freeglut_ext.h>

struct Vector3
{
	double x, y, z;
};

struct Camera
{
	Vector3 pos;
	Vector3 dir;
	Vector3 up;
	double focalLength;
	double aperture;
	double eyeSeparation;
	int screenWidth, screenHeight;
};

EMADevice *z800;

Camera camera;
int win;
float separation = 1.0;
bool isLeftEye;
float fps = 0;
int frameCount, time, timebase=0;

bool doFullscreen = true;
bool doZ800 = true;
float angle;

//---------------------------------------------------------------------------
void Normalize( Vector3 * _v )
{
	double val = _v->x * _v->x + _v->y * _v->y + _v->z * _v->z;
	val = sqrt(val);
	
	if( val )
	{
		_v->x /= val;
		_v->y /= val;
		_v->z /= val;
	}
	else
	{
		printf( "Normalize GOT a ZERO!!!\n" );
	}	
}

//---------------------------------------------------------------------------
void CrossProduct( Vector3 _va, Vector3 _vb, Vector3 * _vr )
{
	_vr->x = _va.y*_vb.z - _va.z*_vb.y;
	_vr->y = _va.z*_vb.x - _va.x*_vb.z;
	_vr->z = _va.x*_vb.y - _va.y*_vb.x;		
}

//---------------------------------------------------------------------------
void SetupLighting()
{
    GLfloat MaterialAmbient[] = {0.5, 0.5, 0.5, 1.0};
    GLfloat MaterialDiffuse[] = {1.0, 1.0, 1.0, 1.0};
    GLfloat MaterialSpecular[] = {1.0, 1.0, 1.0, 1.0};
    GLfloat MaterialShininess[] = {50.0};
    GLfloat AmbientLightPosition[] = {0.5, 1.0, 1.0, 0.0};
    GLfloat LightAmbient[] = {0.5, 0.5, 0.5, 1.0};

    glMaterialfv(GL_FRONT, GL_AMBIENT, MaterialAmbient);
    glMaterialfv(GL_FRONT, GL_DIFFUSE, MaterialDiffuse);
    glMaterialfv(GL_FRONT, GL_SPECULAR, MaterialSpecular);
    glMaterialfv(GL_FRONT, GL_SHININESS, MaterialShininess);
    glLightfv(GL_LIGHT0, GL_POSITION, AmbientLightPosition);
    glLightModelfv(GL_LIGHT_MODEL_AMBIENT, LightAmbient);

    glEnable(GL_LIGHTING);
    glEnable(GL_LIGHT0);
    glEnable(GL_COLOR_MATERIAL);
    glColorMaterial(GL_FRONT, GL_AMBIENT_AND_DIFFUSE);
    glShadeModel(GL_SMOOTH);
}

//---------------------------------------------------------------------------
void doPerspective()
{
	glMatrixMode(GL_PROJECTION);            
	glLoadIdentity();
	
	gluPerspective(camera.aperture,4/3,0.1,1000.0);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
}

//---------------------------------------------------------------------------
void doOrtho()
{
	glMatrixMode(GL_PROJECTION);            
	glLoadIdentity();
	
	gluOrtho2D(0, 800, 0, 600);
	
	// to make text drawing easier, invert the y axis, down is positive
	glScalef(1, -1, 1);
	// mover the origin from the bottom left corner
	// to the upper left corner
	glTranslatef(0, -600, 0);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
}

//---------------------------------------------------------------------------
void displayCallback(void)
{
	Vector3 r, center;
 
	time=glutGet(GLUT_ELAPSED_TIME);
	if (time - timebase > 1000) {
		fps = frameCount*1000.0/(time-timebase);
		timebase = time;		
		frameCount = 0;
	}

	doPerspective();

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
 
	glPushMatrix();

		CrossProduct( camera.dir, camera.up, &r );
		Normalize( &r );
	
		r.x *= camera.eyeSeparation / 2.0;
		r.y *= camera.eyeSeparation / 2.0;
		r.z *= camera.eyeSeparation / 2.0;

		center.x = camera.pos.x + camera.focalLength * camera.dir.x;
		center.y = camera.pos.y + camera.focalLength * camera.dir.y;
		center.z = camera.pos.z + camera.focalLength * camera.dir.z;

		if (isLeftEye)
		{
			gluLookAt(camera.pos.x - r.x,
					camera.pos.y - r.y,
					camera.pos.z - r.z,
					center.x,center.y,center.z,
					camera.up.x,camera.up.y,camera.up.z);
		}
		else
		{
			gluLookAt(camera.pos.x + r.x,
					camera.pos.y + r.y,
					camera.pos.z + r.z,
					center.x,center.y,center.z,
					camera.up.x,camera.up.y,camera.up.z);
		} 
  
		glPushMatrix();
	
			glTranslatef( 0, 0, -2);
			glRotatef( angle, 0, 1, 0 );
 
			glColor3f( 0.2, 0.5, 1.0 );
			glutSolidTorus( 0.3, 0.5, 60, 60 );

  		glPopMatrix();

		// Now draw some text
		doOrtho();
	
		int yPos = 20;
		glColor3f(1, 1, 1);
 		glRasterPos2f(10, yPos);
  		glutBitmapString( GLUT_BITMAP_HELVETICA_18, (const unsigned char*)"Z800 Test Program" );
  	
  		char buffer[80];
  		yPos += 20;
  		snprintf( buffer, 80, "FPS: %4.2f", fps );
 		glRasterPos2f(10, yPos);
  		glutBitmapString( GLUT_BITMAP_HELVETICA_18, (const unsigned char*)buffer );

		if(doZ800)
		{
			uint32_t version;
			z800->GetFirmwareVersion(version);
			yPos += 20;
 		 	snprintf( buffer, 80, "Firmware: %x.%x", (version>>8), (version&0xff) );
 			glRasterPos2f(10, yPos);
	  		glutBitmapString( GLUT_BITMAP_HELVETICA_18, (const unsigned char*)buffer );
		}
	
		yPos += 20;	
  		snprintf( buffer, 80, "Separation: %1.1f", separation );
 		glRasterPos2f(10, yPos);
	  	glutBitmapString( GLUT_BITMAP_HELVETICA_18, (const unsigned char*)buffer );
	
	glPopMatrix();
 
	if(doZ800)
		isLeftEye = !isLeftEye;
	
	glutSwapBuffers();
  	frameCount++;
	angle++;
}

//---------------------------------------------------------------------------
void keyboardCallback(unsigned char key, int x, int y)
{
	if(key == 'q')
	{
		if( doFullscreen )
			glutLeaveGameMode();
		else
			glutDestroyWindow(win);
				
		printf( "Going to sleep..\n." );
		if(doZ800)
			z800->GoSleep();
    	
    	exit(0);
	}
}

//---------------------------------------------------------------------------
void keyboardSpecialCallback( int key, int x, int y )
{
	if( key == GLUT_KEY_DOWN )
	{
		separation -= 0.1;
		if( separation < 0 ) separation = 0;
		camera.eyeSeparation = ( camera.focalLength * separation )/20;
	}
	else if( key == GLUT_KEY_UP )
	{
		separation += 0.1;
		if( separation > 3 ) separation = 3;
		camera.eyeSeparation = ( camera.focalLength * separation )/20;
	}
	else if( key == GLUT_KEY_F1 )
	{
		z800->CycleBrightnessLevel(); 
	}
	else if( key == GLUT_KEY_F2 )
	{
		z800->EnableStereoVision(false);
	}
	else if( key == GLUT_KEY_F3 )
	{
		z800->EnableStereoVision(true);
	}
}

//---------------------------------------------------------------------------
void init3D(int argc, char ** argv)
{
	// initialize glut.  Let it remove any specific
	// commandline options for itself
	glutInit(&argc, argv);

	// now, check for '-noz800' and '-nofullscreen'
	// quick and dirty....
	for( int i=1; i<argc; i++ )
	{
		if( !strcasecmp( argv[i], "-noz800" ) )
		{
			doZ800 = false;
		}
		else if( !strcasecmp( argv[i], "-nofullscreen" ) )
		{
			doFullscreen = false;
		}		
		else
		{
			// invalid option.  Print help
			printf( "Z800 Test program.  Command options:\n" );
			printf( "    -noz800           Don't activate the Z800\n" );
			printf( "    -nofullscreen     Don't go fullscreen\n" );
			exit(0);
		}		
	}
	
	if(doZ800)
	{
		z800 = new EMADevice();
		z800->EnableStereoVision(true);
	}	
	
	isLeftEye = true;
	angle = 0.0f;
	frameCount = 0;
	time = glutGet(GLUT_ELAPSED_TIME);
	
	camera.aperture = 50;
	camera.focalLength = 1.5;
	camera.eyeSeparation = ( camera.focalLength * separation )/20;
 
	camera.pos.x = 0; 
	camera.pos.y = 0; 
	camera.pos.z = camera.focalLength;
	camera.dir.x = 0;
	camera.dir.y = 0;
	camera.dir.z = -1;

	camera.up.x = 0;  
	camera.up.y = 1; 
	camera.up.z = 0;

	// specify the display mode to be RGB and single buffering 
	// we use single buffering since this will be non animated
	glutInitDisplayMode(GLUT_RGBA | GLUT_DOUBLE | GLUT_DEPTH );

	if( !doFullscreen )
	{		
		// define the size
		glutInitWindowSize(800,600);

		// the position where the window will appear
		glutInitWindowPosition(10,10);
  
		// create the window
		win = glutCreateWindow("Yet another demo");
	}
	else
	{
		glutGameModeString( "800x600@60" );
	
		printf( "refresh rate = %d\n", glutGameModeGet(GLUT_GAME_MODE_REFRESH_RATE) );
		if (glutGameModeGet(GLUT_GAME_MODE_POSSIBLE)) 
			glutEnterGameMode();
		else {
			printf("The select mode is not available\n");
			exit(1);	
		}
	}

	glutDisplayFunc(displayCallback);
	glutKeyboardFunc(keyboardCallback);
	glutSpecialFunc(keyboardSpecialCallback);
	glutIdleFunc(displayCallback);
 
	// define the color we use to clearscreen 
	glClearColor(0.0,0.0,0.0,0.0);
	
	glEnable(GL_DEPTH_TEST);

	SetupLighting();
	
	glEnable(GL_CULL_FACE);
	glCullFace(GL_BACK);
		
	// enter the main loop
	glutMainLoop();

	return;
}

//---------------------------------------------------------------------------
int main(int argc, char ** argv)
{
	init3D(argc, argv);
	return 0;
}


